use super::subprocess::{ytdl, ytdl_title};

pub struct SongMetadata {
    pub artist: Option<String>,
    pub title: Option<String>,
    pub duration: Option<u64>,
}

pub struct Song {
    pub url: String,
    metadata: SongMetadata,
}

impl Song {
    pub async fn from_query(query: &str) -> Result<Song, std::io::Error> {
        let query = if query.contains("watch?v=") {
            query.to_string()
        } else {
            format!("ytsearch:{}", query)
        };

        let title = ytdl_title(&query).await?;
        let url = ytdl(&query).await?;

        let metadata = SongMetadata {
            artist: None,
            title: Some(title),
            duration: None,
        };

        let song = Song { url, metadata };
        Ok(song)
    }

    pub async fn get_string(&self) -> String {
        let artist = self
            .metadata
            .artist
            .clone()
            .unwrap_or("unknown".to_string());
        let title = self.metadata.title.clone().unwrap_or("unknown".to_string());
        let duration = match self.metadata.duration {
            Some(duration) => {
                let mins = duration / 60;
                let secs = duration - mins * 60;
                format!("{}:{:0>2}", mins, secs)
            }
            None => "unknown duration".to_string(),
        };
        format!("{} by {} | {}", title, artist, &duration)
    }
}
